/*********************************************************************************
Copyright(c) 2020 Analog Devices, Inc. All Rights Reserved.
This software is proprietary and confidential. By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************/

/*!
* @file      OSPI_Command.c
*
* @brief     This file contains the command structure and other functions for OSPI Octal Flash Direct read access example.
*
*/

#include "main.h"


extern uint8_t STIG_RW_Buffer[8];


/* OSPI STIG command structure for STIG operations */
ADI_OSPI_STIGCMD STIG_Command =  {		/*! command op-code */
										0x3,
										/*! 2nd command byte for commands with extended op-code,
											! Must be set to '0' for all normal commands */
										0,

										/*! Bus mode */
										ADI_OSPI_STIG_SINGLE,

										/*! Transfer mode */
										ADI_OSPI_TRANSFERMODE_STR,

										/*! Number of dummy bytes */
										0,

										/*! Whether Mode data will be sent */
										0,

										/*! Mode byte data */
										0,

										/*! Specifies whether the command has any data requirements */
										ADI_OSPI_STIG_NODATA,

										/*! Number of Data Bytes */
										0,

										/*! address will be sent */
										0,

										/*! Number of Address Bytes */
										ADI_OSPI_CMD_ADDR_SIZE_1,

										/*! flash command address */
										0,
										 /*! Pointer to the data buffer.
										  * Acts as write buffer for commands which send out data
										  * Acts as read buffer for commands which expect data in response
										  */
										STIG_RW_Buffer
									};

void Configure_Octal(void)
{
	*pREG_OSPI0_DRICTL = 0x140333EC;  /* Enabling Octal STR */

	/* Configure the Baudrate */
	/*RCDR Value needs to configured */
#if CONFIG_OCTALBAUD
	*pREG_OSPI0_RDC &= (~BITM_OSPI_RDC_DLYRD)  ;
	*pREG_OSPI0_RDC |= (7 <<BITP_OSPI_RDC_DLYRD);
	*pREG_OSPI0_CTL &= (~BITM_OSPI_CTL_BAUD);
	*pREG_OSPI0_CTL |= ( 0x3<<BITP_OSPI_CTL_BAUD);

#endif

	*pREG_OSPI0_CTL |= (BITM_OSPI_CTL_DACEN | BITM_OSPI_CTL_OPCODEEN); /*Enable the dual opcode mode */
	*pREG_OSPI0_OE_LWR &= (~BITM_OSPI_OE_LWR_XRDBYT);
	*pREG_OSPI0_OE_LWR |= (0x13<< BITP_OSPI_OE_LWR_XRDBYT);
	*pREG_OSPI0_DSCTL &= (~BITM_OSPI_DSCTL_ADDRSZ);
	*pREG_OSPI0_DSCTL |= (0x3 << BITP_OSPI_DSCTL_ADDRSZ); /* Enabling 4 byte address as macrnoix flash expects 4 byte address mode */

}

/*********************************************************************
 Function:       Configure_Flash

 Description:    Configures the Flash device in a mode
 *********************************************************************/
int Configure_Flash(ADI_OSPI_STIGCMD *pSTIG_Command)
{
	int eResult = 0;

	/* Enable OPI mode of Macronix MX66LM1G45G Flash device */
	eResult = OPI_En_MX66LM1G(pSTIG_Command, STR_MODE);
	return eResult;
}


void adi_ospi_StigCommand(ADI_OSPI_STIGCMD *pStigCommand)
{
	ADI_OSPI_REGS* pOSPIRegs = (ADI_OSPI_REGS*)REG_OSPI0_CTL;

	/* Enable required interrupts via the Interrupt Mask Register */
	pOSPIRegs->IRQ_MASK_REG = (uint32_t)ADI_OSPI_INT_MASK;

    /* Configure the bus mode for stig command operation */
    adi_ospi_configure_Stigbusmode(pStigCommand);

    /* Configure the Flash command control register */
    pOSPIRegs->FLASH_CMD_CTRL_REG = 0U;
    pOSPIRegs->FLASH_CMD_CTRL_REG |= (((uint32_t)(pStigCommand->CmdOpcode) << BITP_OSPI_FCCTL_OPCODE) &
                                                                              BITM_OSPI_FCCTL_OPCODE) |
                                     (((uint32_t)(pStigCommand->NumOfDummyCycles) << BITP_OSPI_FCCTL_DMY) &
                                                                                     BITM_OSPI_FCCTL_DMY);
    /* Check if this command requires address */
    if(pStigCommand->AddressEnable == 1U)
    {
        pOSPIRegs->FLASH_CMD_CTRL_REG |= ((pStigCommand->AddressEnable   << BITP_OSPI_FCCTL_ADDREN) &
                                                                            BITM_OSPI_FCCTL_ADDREN) |
                                         (((((uint32_t)pStigCommand->AddrSize )-1U) << BITP_OSPI_FCCTL_ADDRSZ) &
                                                                                       BITM_OSPI_FCCTL_ADDRSZ);
        pOSPIRegs->FLASH_CMD_ADDR_REG = pStigCommand->Address;
    }

    /* Check if this command reads back data from flash device */
    if(pStigCommand->eDataMode == ADI_OSPI_STIG_READDATA_EN)
    {
        pOSPIRegs->FLASH_CMD_CTRL_REG |= (( 1UL << BITP_OSPI_FCCTL_RDEN) &
                                                  BITM_OSPI_FCCTL_RDEN) |
                                         ((((uint32_t)(pStigCommand->DataSize)-1U) << BITP_OSPI_FCCTL_RDSZ) &
                                                                                      BITM_OSPI_FCCTL_RDSZ);
    }

    /* Check if this command requires data to write to flash device */
    if(pStigCommand->eDataMode == ADI_OSPI_STIG_WRITEDATA_EN)
    {
        pOSPIRegs->FLASH_CMD_CTRL_REG |= (( 1UL << BITP_OSPI_FCCTL_WREN) &
                                                  BITM_OSPI_FCCTL_WREN) |
                                         ((((uint32_t)(pStigCommand->DataSize)-1U)  << BITP_OSPI_FCCTL_WRSZ) &
                                                                                       BITM_OSPI_FCCTL_WRSZ);
    }

    /* Check if mode data needs to be send with the command */
    if(pStigCommand->ModeDataEnable == 1U)
    {
        pOSPIRegs->FLASH_CMD_CTRL_REG |= (( 1UL << BITP_OSPI_FCCTL_MODEEN) &
                                                  BITM_OSPI_FCCTL_MODEEN);

        /* Configure the modedata value */
        pOSPIRegs->MODE_BIT_CONFIG_REG &= ~BITM_OSPI_MBCTL_MODE;
        pOSPIRegs->MODE_BIT_CONFIG_REG |= ((uint32_t)(pStigCommand->ModeData) << BITP_OSPI_MBCTL_MODE) &
                                                                                 BITM_OSPI_MBCTL_MODE;
    }

    /* Check if the command has extended opcode */
    if(pStigCommand->CmdOpcode2 != 0U)
    {
        /* Configure the extended opcode resgister */
        pOSPIRegs->OPCODE_EXT_LOWER_REG &= ~BITM_OSPI_OE_LWR_XSTIGBYT;
        pOSPIRegs->OPCODE_EXT_LOWER_REG |= ((uint32_t)(pStigCommand->CmdOpcode2) << BITP_OSPI_OE_LWR_XSTIGBYT) &
                                                                                    BITM_OSPI_OE_LWR_XSTIGBYT;

        /* Enable dual op-code mode */
        pOSPIRegs->CONFIG_REG |= BITM_OSPI_CTL_OPCODEEN;
    }
    else
    {
        /* Make sure the dual op-code is not enabled */
        pOSPIRegs->CONFIG_REG &= ~BITM_OSPI_CTL_OPCODEEN;
    }

    /* Load data if the command expects the data to be written out */
    if(pStigCommand->eDataMode == ADI_OSPI_STIG_WRITEDATA_EN)
    {
        uint8_t nIndex = 0U;
        uint8_t *pBuffer = pStigCommand->pBuffer;
        uint32_t nData_Lo = 0U, nData_Hi = 0U;

        while(nIndex < pStigCommand->DataSize)
        {
            if(nIndex < 4U)
            {
                nData_Lo |= (uint32_t)((uint32_t)(*pBuffer++) << (8U*nIndex));
            }
            else
            {
                nData_Hi |= (uint32_t)((uint32_t)(*pBuffer++) << (8U*(nIndex-4U)));
            }

            nIndex++;
        }

        /* Write the requested data in STIG write registers */
        pOSPIRegs->FLASH_WR_DATA_LOWER_REG = nData_Lo;
        pOSPIRegs->FLASH_WR_DATA_UPPER_REG = nData_Hi;
    }

    /* Launch the command */
    pOSPIRegs->FLASH_CMD_CTRL_REG |= BITM_OSPI_FCCTL_EXE;

    while(pOSPIRegs->IRQ_STATUS_REG != BITM_OSPI_ISTAT_STIGREQ);

    pOSPIRegs->IRQ_STATUS_REG = BITM_OSPI_ISTAT_STIGREQ;
    /* If the command expects the data, read it into the buffer */
    if(pStigCommand->eDataMode == ADI_OSPI_STIG_READDATA_EN)
    {
        uint8_t nIndex = 0U;
        uint8_t *pBuffer = pStigCommand->pBuffer;
        uint32_t nData_Lo = 0U, nData_Hi = 0U;

        /* Put the read data into user passed buffer */
        nData_Lo = pOSPIRegs->FLASH_RD_DATA_LOWER_REG;
        nData_Hi = pOSPIRegs->FLASH_RD_DATA_UPPER_REG;

        while(nIndex < pStigCommand->DataSize)
        {
            if(nIndex  < 4U)
            {
                *pBuffer++ = (uint8_t)((nData_Lo >> (8U*nIndex)) & 0xFFU);
            }
            else
            {
                *pBuffer++ = (uint8_t)((nData_Hi >> ((8U*(nIndex-4U)))) & 0xFFU);
            }

            nIndex++;
        }
    }
}

void adi_ospi_configure_Stigbusmode(ADI_OSPI_STIGCMD * pStigCommand)
{
    ADI_OSPI_REGS* pOSPIRegs = (ADI_OSPI_REGS*)REG_OSPI0_CTL;
    /* Clear the contents of the Read instruction register */
    pOSPIRegs->DEV_INSTR_RD_CONFIG_REG = 0U;


    /* Configure the bus width as per the user passed bus mode */
    switch(pStigCommand->eBusMode)
    {
        case ADI_OSPI_STIG_SINGLE:
            pOSPIRegs->DEV_INSTR_RD_CONFIG_REG |= (0UL << BITP_OSPI_DRICTL_INSTRTYP);
            break;
        case ADI_OSPI_STIG_DPI:
            pOSPIRegs->DEV_INSTR_RD_CONFIG_REG |= (1UL << BITP_OSPI_DRICTL_INSTRTYP);
            break;
        case ADI_OSPI_STIG_QPI:
            pOSPIRegs->DEV_INSTR_RD_CONFIG_REG |= (2UL << BITP_OSPI_DRICTL_INSTRTYP);
            break;
        case ADI_OSPI_STIG_OPI:
            pOSPIRegs->DEV_INSTR_RD_CONFIG_REG |= (3UL << BITP_OSPI_DRICTL_INSTRTYP);
            break;

    }

    /* Configure the bus operation STR/DTR */
     adi_ospi_configure_TransferMode(pStigCommand->eTransferMode);

}



void adi_ospi_configure_TransferMode(ADI_OSPI_TRANSFERMODE eTransferMode)
{

    ADI_OSPI_REGS* pOSPIRegs = (ADI_OSPI_REGS*)REG_OSPI0_CTL;

    /* Clear the DTR mode bits */
    pOSPIRegs->DEV_INSTR_RD_CONFIG_REG &= ~(BITM_OSPI_DRICTL_DDREN);
    pOSPIRegs->CONFIG_REG &= ~(BITM_OSPI_CTL_DTREN);

    switch(eTransferMode)
    {
        case ADI_OSPI_TRANSFERMODE_STR:
            pOSPIRegs->DEV_INSTR_RD_CONFIG_REG |= (0UL << BITP_OSPI_DRICTL_DDREN);
            break;
        case ADI_OSPI_TRANSFERMODE_DTR:
            pOSPIRegs->DEV_INSTR_RD_CONFIG_REG |= (1UL << BITP_OSPI_DRICTL_DDREN);
            break;
        case ADI_OSPI_TRANSFERMODE_DTR_P:
            pOSPIRegs->DEV_INSTR_RD_CONFIG_REG |= (1UL << BITP_OSPI_DRICTL_DDREN);
            pOSPIRegs->CONFIG_REG |= (1UL << BITP_OSPI_CTL_DTREN);
            break;
    }

}


/*********************************************************************
 Function:       OPI_En_MX66LM1G

 Description:    Enables the OPI mode of MX66LM1G flash device
 *********************************************************************/
int OPI_En_MX66LM1G(ADI_OSPI_STIGCMD *pSTIG_Command, FLASH_OPI_MODE Transfer_Mode)
{
	uint8_t Flash_Status[1];
	uint8_t Flash_WRCR2[1];

	/* Enable the Write to flash registers by sending FLASH_WR_EN(0x6) command */
	pSTIG_Command->CmdOpcode       = FLASH_WR_EN;
	pSTIG_Command->CmdOpcode2      = 0;
	pSTIG_Command->eBusMode        = ADI_OSPI_STIG_SINGLE;
	pSTIG_Command->eTransferMode   = ADI_OSPI_TRANSFERMODE_STR;
	pSTIG_Command->NumOfDummyCycles= 0;
	pSTIG_Command->ModeDataEnable  = 0;
	pSTIG_Command->ModeData		   = 0;
	pSTIG_Command->eDataMode	   = ADI_OSPI_STIG_NODATA;
	pSTIG_Command->DataSize		   = 1;
	pSTIG_Command->AddressEnable   = 0;
	pSTIG_Command->AddrSize		   = ADI_OSPI_CMD_ADDR_SIZE_1;
	pSTIG_Command->Address         = 0;
	pSTIG_Command->pBuffer         = 0;

	adi_ospi_StigCommand(pSTIG_Command);

	/* Initialize the STIG command to read Flash status */
	pSTIG_Command->CmdOpcode       = FLASH_RD_STAT1;
	pSTIG_Command->eDataMode	   = ADI_OSPI_STIG_READDATA_EN;
	pSTIG_Command->DataSize		   = 1;
	pSTIG_Command->pBuffer		   = Flash_Status;

	/* Initialize the STIG buffer */
	Flash_Status[0] = 0;

	/* Wait till WEL bit in flash status register is not set */
	while(!(Flash_Status[0] & WEL) )
	{
		adi_ospi_StigCommand(pSTIG_Command);
	}


	/* Initialize the STIG command to write to MX66LM1G WRCR2 register */
	pSTIG_Command->CmdOpcode       = FLASH_WR_WRCR2;
	pSTIG_Command->eDataMode	   = ADI_OSPI_STIG_WRITEDATA_EN;
	pSTIG_Command->DataSize		   = 1;
	pSTIG_Command->AddressEnable   = 1;
	pSTIG_Command->AddrSize		   = ADI_OSPI_CMD_ADDR_SIZE_4;
	pSTIG_Command->Address         = 0;
	pSTIG_Command->pBuffer		   = Flash_WRCR2;

	/* Update the value to be written to WRCR2 register */
	if(Transfer_Mode == STR_MODE)
	{
		Flash_WRCR2[0] =  0x1;
	}
	else
	{
		Flash_WRCR2[0] =  0x2;
	}

	adi_ospi_StigCommand(pSTIG_Command);

	/* Put some delay here */
	int Delay = 0;
	while(Delay++ < 10000){ asm("nop;"); }

	/* Read back the Config2 Register in STT_OPI mode */
	pSTIG_Command->CmdOpcode       = FLASH_RD_WRCR2;
	pSTIG_Command->CmdOpcode2      = 0x8E;
	pSTIG_Command->eBusMode        = ADI_OSPI_STIG_OPI;
	if(Transfer_Mode == STR_MODE)
	{
		pSTIG_Command->eTransferMode   = ADI_OSPI_TRANSFERMODE_STR;
	}
	else
	{
		pSTIG_Command->eTransferMode   = ADI_OSPI_TRANSFERMODE_DTR_P;
	}
	pSTIG_Command->NumOfDummyCycles= 20;
	pSTIG_Command->ModeDataEnable  = 0;
	pSTIG_Command->ModeData		   = 0;
	pSTIG_Command->eDataMode	   = ADI_OSPI_STIG_READDATA_EN;
	pSTIG_Command->DataSize		   = 1;
	pSTIG_Command->AddressEnable   = 1;
	pSTIG_Command->AddrSize		   = ADI_OSPI_CMD_ADDR_SIZE_4;
	pSTIG_Command->Address         = 0;
	pSTIG_Command->pBuffer         = Flash_WRCR2;

	Flash_WRCR2[0] = 0;

	adi_ospi_StigCommand(pSTIG_Command);

	if(Transfer_Mode == STR_MODE)
	{
		if(Flash_WRCR2[0] !=  0x1)
		{
			return 1;
		}
	}
	else
	{
		if(Flash_WRCR2[0] !=  0x2)
		{
			return 1;
		}
	}

	return 0;
}
